'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var dayjs = require('dayjs');
var vueScrollPicker = require('vue-scroll-picker');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var dayjs__default = /*#__PURE__*/_interopDefaultLegacy(dayjs);

var DrumrollSeparator = {
  name: 'DrumrollSeparator',

  functional: true,

  props: {
    separator: { type: String, required: true },
  },

  render (h, { props }) {
    return h('span', { class: 'v-drumroll-picker__separator -selected' }, [props.separator])
  },
};

var useSensitivity = {
  props: {
    /**
     * Dependency library properties
     * @see https://github.com/wan2land/vue-scroll-picker
     */
    dragSensitivity: { type: [String, Number], default: 1.7 },
    touchSensitivity: { type: [String, Number], default: 1.7 },
    scrollSensitivity: { type: [String, Number], default: 0.8 },
  },
};

var BasePicker = {
  name: 'BasePicker',

  functional: true,

  mixins: [
    useSensitivity,
  ],

  props: {
    items: { type: Array, required: true },
    format: { type: String, required: true },
    height: { type: [String, Number], default: '10em' },
    maxDate: { type: [String, Number, Date], default: undefined },
    minDate: { type: [String, Number, Date], required: true },
    unit: { type: String, required: true },
    value: { type: [String, Number], default: undefined },
    width: { type: [String, Number], default: '2em' },
  },

  render (h, { props, listeners }) {
    return h(vueScrollPicker.ScrollPicker, {
      style: {
        height: typeof props.height === 'string' ? props.height : props.height + 'px',
        width: typeof props.width === 'string' ? props.width : props.width + 'px',
      },
      props: {
        options: props.items,
        dragSensitivity: props.dragSensitivity,
        touchSensitivity: props.touchSensitivity,
        scrollSensitivity: props.scrollSensitivity,
        value: dayjs__default["default"](props.value).get(props.unit),
      },
      on: {
        input: value => {
          if (!value) value = 0;

          const day = dayjs__default["default"](props.value, props.format);
          const current = day.get(props.unit);
          const date = day.set(props.unit, value);

          // 桁上がり抑止
          if (current <= value && date.get(props.unit) < value) return
          listeners.input(date.format(props.format));
        },
      },
    })
  },
};

const DEFAULT_MIN_DATE = '1900-01-01 00:00:00';

const MONTH_UNIT = 12;
const HOUR_UNIT = 24;
const MINUTE_UNIT = 60;
const DIGIT = 2;

var DatePicker$1 = {
  name: 'BaseDatePicker',

  mixins: [useSensitivity],

  props: {
    defaultValue: { type: String, default: undefined },
    format: { type: String, default: 'YYYY-MM-DD' },
    height: { type: [String, Number], default: undefined },
    maxDate: { type: [String, Number, Date], default: undefined },
    minDate: { type: [String, Number, Date], default: DEFAULT_MIN_DATE },
    separator: { type: String, required: true },
    value: { type: [String, Number, Date], default: undefined },
  },

  data () {
    const date = dayjs__default["default"](this.value || this.defaultValue).endOf('month').date();
    return {
      date: date,
      numberOfDays: date,
      dateOfMin: 1,
    }
  },

  computed: {
    /**
     * 年配列
     *
     * @return {array}
     */
    years () {
      const minDate = dayjs__default["default"](this.minDate);
      const maxYear = this.maxDate
        ? dayjs__default["default"](this.maxDate).year()
        : dayjs__default["default"](this.value || this.defaultValue, this.format).add(100, 'year').year();

      const years = [];
      for (let year = minDate.year(); year <= maxYear; year++) {
        years.push(year);
      }

      return years
    },

    /**
     * 月配列
     *
     * @return {array}
     */
    months () {
      const currentDate = dayjs__default["default"](this.value || this.defaultValue, this.format);
      const minDate = dayjs__default["default"](this.minDate);
      let min = currentDate.isSame(minDate, 'year') ? minDate.format('M') : 1;
      let max = this.maxDate && currentDate.isSame(this.maxDate, 'year')
        ? dayjs__default["default"](this.maxDate).format('M')
        : MONTH_UNIT;

      if (min > currentDate.format('M') || max < currentDate.format('M')) {
        min = 1;
        max = MONTH_UNIT;
      }

      // 桁揃えをしつつ時刻を配列に追加
      const months = [];
      for (let month = min; month <= max; month++) {
        months.push({
          name: month.toString().padStart(DIGIT, '0'),
          value: month - 1,
        });
      }

      return months
    },

    /**
     * 日配列
     *
     * @params {string} date
     * @return {array}
     */
    days () {
      const currentDate = dayjs__default["default"](this.value || this.defaultValue, this.format);
      const minDate = dayjs__default["default"](this.minDate);
      let min = currentDate.isSame(minDate, 'month') ? minDate.date() : 1;
      let max = this.maxDate && currentDate.isSame(this.maxDate, 'month')
        ? dayjs__default["default"](this.maxDate).date()
        : this.numberOfDays;

      if (min > currentDate.date() || max < currentDate.date()) {
        min = 1;
        max = this.numberOfDays;
      }

      // 桁揃えをしつつ時刻を配列に追加
      const days = [];
      for (let date = Math.min(this.dateOfMin, min); date <= max; date++) {
        days.push({
          name: date <= this.date && date >= min ? date.toString().padStart(DIGIT, '0') : '',
          value: date,
        });
      }

      this.$nextTick(() => setTimeout(() => {
        this.dateOfMin = min;
        if (this.date !== this.numberOfDays) {
          this.numberOfDays = this.date;
        }
      }, 100));

      return days
    },
  },

  watch: {
    value (newValue) {
      const newDate = dayjs__default["default"](newValue);
      const lastDate = newDate.endOf('month').date();
      if (lastDate !== this.date) this.date = lastDate;
    },
  },

  methods: {
    onInput (value) {
      const defaultValue = dayjs__default["default"](this.defaultValue).format(this.format);
      if (dayjs__default["default"](value).isValid() && (this.value || value !== defaultValue)) {
        this.$emit('input', value);
      }
    },
  },

  render (h) {
    const separator = h(DrumrollSeparator, { props: { separator: this.separator } });

    const yearPicker = h(BasePicker, {
      props: {
        items: this.years,
        unit: 'year',
        width: DIGIT * 1.5 + 'em',
        ...this.$props,
        value: this.value || this.defaultValue,
      },
      on: {
        input: this.onInput,
      },
    });

    const monthPicker = h(BasePicker, {
      props: {
        items: this.months,
        unit: 'month',
        width: DIGIT + 'em',
        ...this.$props,
        value: this.value || this.defaultValue,
      },
      on: {
        input: this.onInput,
      },
    });

    const dayPicker = h(BasePicker, {
      props: {
        items: this.days,
        unit: 'date',
        width: DIGIT + 'em',
        ...this.$props,
        value: this.value || this.defaultValue,
      },
      on: {
        input: this.onInput,
      },
    });

    return h(vueScrollPicker.ScrollPickerGroup, { class: 'vdd-flex' }, [
      yearPicker,
      separator,
      monthPicker,
      separator,
      dayPicker,
    ])
  },
};

/**
 * Format date string
 *
 * @param {string|number|Date} value
 * @param {string|undefined} modelFormat
 * @param {string} type
 * @return {string|undefined}
 */
var datestring = (value, modelFormat, type) => {
  if (!value) return undefined

  const baseFormat = '^' + modelFormat
    .replace(/(Aa|M{3,4}|d{2,4})/g, '[\\u\\l]+?')
    .replace(/([YMDHms])/g, '\\d');
  const format = new RegExp(baseFormat);
  if (format.test(value)) {
    const modelValue = dayjs__default["default"](value, modelFormat);
    if (modelValue.isValid()) {
      return modelValue.format(modelFormat)
    } else if (type === 'time' && typeof value === 'string') {
      const datetime = `1980-01-01 ${value}`;
      const date = dayjs__default["default"](datetime, `YYYY-MM-DD ${modelFormat}`);
      return date.isValid() ? datetime : undefined
    }
    return undefined
  } else {
    return undefined
  }
};

var TimePicker$1 = {
  name: 'BaseTimePicker',

  mixins: [useSensitivity],

  props: {
    defaultValue: { type: String, default: undefined },
    format: { type: String, default: 'YYYY-MM-DD HH:mm' },
    height: { type: [String, Number], default: undefined },
    maxDate: { type: [String, Number, Date], default: undefined },
    minDate: { type: [String, Number, Date], default: undefined },
    minuteInterval: { type: [String, Number], default: 1 },
    separator: { type: String, required: true },
    value: { type: [String, Number, Date], default: undefined },
  },

  computed: {
    /**
     * 時配列
     *
     * @return {array}
     */
    hours () {
      let min = 0;
      let max = HOUR_UNIT;

      const currentDate = dayjs__default["default"](this.value);

      if (this.minDate) {
        const minDate = dayjs__default["default"](this.minDate);
        min = currentDate.isSame(minDate, 'date') ? minDate.hour() : 0;
      }
      if (this.maxDate) {
        max = this.maxDate && currentDate.isSame(this.maxDate, 'date')
          ? dayjs__default["default"](this.maxDate).hour() + 1 : HOUR_UNIT;
      }

      const hours = [];
      for (let time = min; time < max; time++) hours.push(('0' + time).slice(-DIGIT));
      return hours
    },

    /**
     * 分配列
     *
     * @return {array}
     */
    minutes () {
      const currentDate = dayjs__default["default"](this.value);
      let min = 0;
      let max = MINUTE_UNIT;

      if (this.minDate) {
        const minDate = dayjs__default["default"](this.minDate);
        min = currentDate.isSame(minDate, 'hour') ? minDate.minute() : 0;
      }
      if (this.maxDate) {
        max = this.maxDate && currentDate.isSame(this.maxDate, 'hour')
          ? dayjs__default["default"](this.maxDate).minute() + 1
          : MINUTE_UNIT;
      }

      const interval = Number(this.minuteInterval);
      const minutes = [];
      for (let minute = min; minute < max; minute += interval) {
        minutes.push(('0' + minute).slice(-DIGIT));
      }
      return minutes
    },
  },

  methods: {
    onInput (value) {
      const modelValue = datestring(value, this.format, 'time');
      const defaultValue = datestring(this.defaultValue || dayjs__default["default"]().format(this.format), this.format, 'time');
      if (modelValue && (this.value || modelValue !== defaultValue)) this.$emit('input', dayjs__default["default"](modelValue).format(this.format));
    },
  },

  render (h) {
    // 境界線
    const separator = h(DrumrollSeparator, { props: { separator: this.separator } });

    // 時
    const hourPicker = h(BasePicker, {
      props: {
        ...this.$props,
        items: this.hours,
        unit: 'hour',
        width: DIGIT + 'em',
      },
      on: {
        input: this.onInput,
      },
    });

    // 分
    const minutePicker = h(BasePicker, {
      props: {
        ...this.$props,
        items: this.minutes,
        unit: 'minute',
        width: DIGIT + 'em',
      },
      on: {
        input: this.onInput,
      },
    });

    return h(vueScrollPicker.ScrollPickerGroup, { class: 'vdd-flex' }, [
      hourPicker,
      separator,
      minutePicker,
    ])
  },
};

// Material Design Icons v5.9.55
var mdiCheckBold = "M9,20.42L2.79,14.21L5.62,11.38L9,14.77L18.88,4.88L21.71,7.71L9,20.42Z";

var ConfirmButton = {
  name: 'ConfirmButton',

  functional: true,

  props: {
    size: { type: [Number, String], default: 21 },
  },

  render (h, { props, listeners }) {
    const icon = h('svg',
      {
        class: ['v-drumroll-picker__button--icon'],
        style: {
          height: `${props.size}px`,
          width: `${props.size}px`,
        },
        attrs: {
          viewBox: '0 0 24 24',
        },
      },
      [h('path', {
        attrs: { d: mdiCheckBold },
      })],
    );

    return h('button', {
      class: ['v-drumroll-picker__button'],
      on: { ...listeners },
    }, [icon])
  },
};

const pickerLayer = h => (
  h('div', { class: ['vue-scroll-picker-layer'] }, [
    h('div', { class: ['top'], ref: 'top' }),
    h('div', { class: ['middle'], ref: 'selection' }),
    h('div', { class: ['bottom'], ref: 'bottom' }),
  ])
);

const confirmButton = (h, { props, listeners }) => (
  !props.dialog || props.hideButton
    ? [] : [h(ConfirmButton, { on: listeners })]
);

var PickerContainer = {
  name: 'PickerContainer',

  functional: true,

  props: {
    dialog: { type: Boolean, default: false },
    height: { type: [Number, String], default: '10em' },
    hideButton: { type: Boolean, default: false },
  },

  render: (h, context) => (h('div',
    {
      class: 'v-drumroll-picker__container',
      style: {
        height: typeof context.props.height === 'string'
          ? context.props.height : context.props.height + 'px',
      },
    },
    [
      pickerLayer(h),
      ...context.children,
      pickerLayer(h),
      ...confirmButton(h, context),
    ],
  )),
};

var useBindings = {
  props: {
    defaultValue: { type: String, default: undefined },
    format: { type: String, default: undefined },
    value: { type: [String, Number, Date], default: undefined },
  },
  computed: {
    modelValue: {
      get () {
        const modelValue = this.value || this.defaultValue || dayjs__default["default"]().format(this.modelFormat);
        return datestring(modelValue, this.modelFormat, this.type)
      },
      set (value) {
        this.$emit('input', dayjs__default["default"](value).format(this.modelFormat));
      },
    },
    modelFormat () {
      if (this.format) return this.format
      switch (this.type) {
        case 'datetime': return 'YYYY-MM-DD HH:mm'
        case 'date': return 'YYYY-MM-DD'
        case 'time': return 'HH:mm'
        default: throw new Error('Invalid property. "type" is only allow "datetime/date/time".')
      }
    },
  },
  methods: {
    onInput (value) {
      if (dayjs__default["default"](value).isBefore(this.minDate)) {
        this.modelValue = datestring(this.minDate, this.modelFormat, this.type);
      } else if (this.maxDate && dayjs__default["default"](value).isAfter(this.maxDate)) {
        this.modelValue = datestring(this.maxDate, this.modelFormat, this.type);
      } else {
        this.modelValue = datestring(value, this.modelFormat, this.type);
      }
    },

    onNativeInput (event) {
      const value = datestring(event.target.value, this.modelFormat, this.type);
      if (value) this.modelValue = value;
    },
  },
};

var OverlayLayer = {
  name: 'OverlayLayer',
  functional: true,
  props: {
    dark: { type: Boolean, default: false },
  },
  render (h, context) {
    return h('div', {
      class: [
        'v-drumroll-picker__overlay',
        context.props.dark ? '--dark' : '',
      ],
      on: {
        click: context.listeners.click,
      },
    })
  },
};

const transitionLayer = (h, content) => (
  h('transition', {
    props: {
      name: 'v-drumroll-picker__slide-transition',
      mode: 'in-out',
      appear: true,
      duration: 300,
    },
  }, [content])
);

var ContentLayer = {
  name: 'ContentLayer',
  functional: true,
  render (h, context) {
    return transitionLayer(h, h('div', {
      class: 'v-drumroll-picker__content',
      on: { click: e => e.stopPropagation() },
    }, context.children))
  },
};

const disableScroll = (e) => e.preventDefault();

var useDialog = {
  props: {
    dialog: { type: Boolean, default: false },
    height: { type: [String, Number], default: undefined },
    hideOverlay: { type: Boolean, default: false },
    hideButton: { type: Boolean, default: false },
  },

  data () {
    return {
      active: false,
    }
  },

  methods: {
    generateDialogPicker (h) {
      const content = [this.generateActivator(h)];

      if (this.active) {
        // overlay
        content.push(h(OverlayLayer, {
          props: { dark: !this.hideOverlay },
          on: { click: this.offActivate },
        }));

        // picker
        const picker = h(PickerContainer, {
          props: this.$props,
          on: { click: this.offActivate },
        }, [this.pickers(h)]);
        content.push(h(ContentLayer, [picker]));
      }
      return h('div', { class: ['v-drumroll-picker', 'v-drumroll-picker--dialog'] }, content)
    },

    generateActivator (h) {
      const on = {
        click: this.onActivate,
        focus: this.onActivate,
        touchend: this.onActivate,
      };

      if (this.$scopedSlots.activator) {
        return this.$scopedSlots.activator({ on })
      }

      // Fallback default
      const options = {
        class: 'v-drumroll-picker__input',
        domProps: { value: this.value },
        on: { input: this.onNativeInput, ...on },
      };
      return h('input', options)
    },

    /**
     * @param {KeyboardEvent} e
     */
    onClose (e) {
      if (e.key.toUpperCase() === 'ESCAPE') this.offActivate(e);
    },

    /**
     * @param {MouseEvent} e
     */
    onActivate (e) {
      e.preventDefault();

      // Close on ESC
      document.addEventListener('keydown', this.onClose);
      document.addEventListener('touchmove', disableScroll, { passive: false });

      // Blur active element.
      const isElement = document.activeElement instanceof HTMLElement;
      if (!this.active && !this.hideOverlay && isElement) document.activeElement.blur();

      // Emit initial value
      const modelFormat = this.format || this.modelFormat;
      const modelValue = this.value || this.defaultValue || dayjs__default["default"]().format(modelFormat);
      const initialValue = datestring(modelValue, modelFormat, this.type);
      this.$emit('input', initialValue);

      this.active = !this.active;
    },

    /**
     * @param {MouseEvent|KeyboardEvent} e
     */
    offActivate (e) {
      e.preventDefault();
      document.removeEventListener('keydown', this.onClose);
      document.removeEventListener('touchmove', disableScroll);
      this.active = false;
    },
  },
};

const generateDatePicker = (h, options) => (
  h(DatePicker$1, options({ separator: '-' }))
);

const generateTimePicker = (h, options) => (
  h(TimePicker$1, options({ separator: ':' }))
);

var DateTimePicker = {
  name: 'DateTimePicker',

  mixins: [
    useBindings,
    useDialog,
    useSensitivity,
  ],

  props: {
    height: { type: [String, Number], default: undefined },
    hideButton: { type: Boolean, default: false },
    maxDate: { type: [String, Number, Date], default: undefined },
    minDate: { type: [String, Number, Date], default: () => DEFAULT_MIN_DATE },
    minuteInterval: { type: [String, Number], default: 1 },
    type: { type: String, default: 'datetime' },
  },

  methods: {

    pickers (h) {
      const options = props => ({
        props: {
          ...this.$props,
          ...props,
          value: this.modelValue,
          format: this.modelFormat,
        },
        on: { input: this.onInput },
      });

      switch (this.type) {
        case 'datetime': return [generateDatePicker(h, options), generateTimePicker(h, options)]
        case 'date': return [generateDatePicker(h, options)]
        case 'time': return [generateTimePicker(h, options)]
      }
    },
  },

  render (h) {
    if (this.dialog) {
      return this.generateDialogPicker(h)
    } else {
      const container = h(PickerContainer, { props: this.$props }, [this.pickers(h)]);
      return h('div', { class: ['v-drumroll-picker'] }, [container])
    }
  },
};

var DatePicker = {
  name: 'DatePicker',

  mixins: [
    useBindings,
    useDialog,
    useSensitivity,
  ],

  props: {
    height: { type: [String, Number], default: undefined },
    hideButton: { type: Boolean, default: false },
    maxDate: { type: [String, Number, Date], default: undefined },
    minDate: { type: [String, Number, Date], default: () => DEFAULT_MIN_DATE },
    separator: { type: String, default: '-' },
  },

  computed: {
    type () {
      return 'date'
    },
  },

  methods: {
    pickers (h) {
      const options = () => ({
        props: this.$props,
        on: { input: this.onInput },
      });

      return [h(DatePicker$1, options())]
    },
  },

  render (h) {
    if (this.dialog) {
      return this.generateDialogPicker(h)
    } else {
      const props = {
        ...this.$props,
        value: this.modelValue,
      };
      const container = h(PickerContainer, { props }, [this.pickers(h)]);
      return h('div', { class: ['v-drumroll-picker'] }, [container])
    }
  },
};

var TimePicker = {
  name: 'TimePicker',

  mixins: [
    useBindings,
    useDialog,
    useSensitivity,
  ],

  props: {
    height: { type: [String, Number], default: undefined },
    hideButton: { type: Boolean, default: false },
    minuteInterval: { type: [String, Number], default: 1 },
    separator: { type: String, default: ':' },
  },

  computed: {
    type () {
      return 'time'
    },
  },

  methods: {
    onInput (value) {
      this.modelValue = datestring(value, this.modelFormat, this.type);
    },

    pickers (h) {
      const options = () => ({
        props: this.$props,
        on: { input: this.onInput },
      });

      return [h(TimePicker$1, options())]
    },
  },

  render (h) {
    if (this.dialog) {
      return this.generateDialogPicker(h)
    } else {
      const props = {
        ...this.$props,
        value: this.modelValue,
        format: this.modelFormat,
      };
      const container = h(PickerContainer, { props }, [this.pickers(h)]);
      return h('div', { class: ['v-drumroll-picker'] }, [container])
    }
  },
};

exports.DatePicker = DatePicker;
exports.TimePicker = TimePicker;
exports["default"] = DateTimePicker;
